"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .responsesimagegenerationcall import (
    ResponsesImageGenerationCall,
    ResponsesImageGenerationCallTypedDict,
)
from .responsesoutputitemfilesearchcall import (
    ResponsesOutputItemFileSearchCall,
    ResponsesOutputItemFileSearchCallTypedDict,
)
from .responsesoutputitemfunctioncall import (
    ResponsesOutputItemFunctionCall,
    ResponsesOutputItemFunctionCallTypedDict,
)
from .responsesoutputitemreasoning import (
    ResponsesOutputItemReasoning,
    ResponsesOutputItemReasoningTypedDict,
)
from .responsesoutputmessage import (
    ResponsesOutputMessage,
    ResponsesOutputMessageTypedDict,
)
from .responseswebsearchcalloutput import (
    ResponsesWebSearchCallOutput,
    ResponsesWebSearchCallOutputTypedDict,
)
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


ResponsesOutputItemTypedDict = TypeAliasType(
    "ResponsesOutputItemTypedDict",
    Union[
        ResponsesWebSearchCallOutputTypedDict,
        ResponsesOutputItemFileSearchCallTypedDict,
        ResponsesImageGenerationCallTypedDict,
        ResponsesOutputMessageTypedDict,
        ResponsesOutputItemFunctionCallTypedDict,
        ResponsesOutputItemReasoningTypedDict,
    ],
)
r"""An output item from the response"""


ResponsesOutputItem = Annotated[
    Union[
        Annotated[ResponsesOutputMessage, Tag("message")],
        Annotated[ResponsesOutputItemReasoning, Tag("reasoning")],
        Annotated[ResponsesOutputItemFunctionCall, Tag("function_call")],
        Annotated[ResponsesWebSearchCallOutput, Tag("web_search_call")],
        Annotated[ResponsesOutputItemFileSearchCall, Tag("file_search_call")],
        Annotated[ResponsesImageGenerationCall, Tag("image_generation_call")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]
r"""An output item from the response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatgenerationtokenusage import (
    ChatGenerationTokenUsage,
    ChatGenerationTokenUsageTypedDict,
)
from .chatresponsechoice import ChatResponseChoice, ChatResponseChoiceTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from openrouter.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ChatResponseTypedDict(TypedDict):
    id: str
    choices: List[ChatResponseChoiceTypedDict]
    created: float
    model: str
    object: Literal["chat.completion"]
    system_fingerprint: NotRequired[Nullable[str]]
    usage: NotRequired[ChatGenerationTokenUsageTypedDict]


class ChatResponse(BaseModel):
    id: str

    choices: List[ChatResponseChoice]

    created: float

    model: str

    OBJECT: Annotated[
        Annotated[
            Literal["chat.completion"],
            AfterValidator(validate_const("chat.completion")),
        ],
        pydantic.Field(alias="object"),
    ] = "chat.completion"

    system_fingerprint: OptionalNullable[str] = UNSET

    usage: Optional[ChatGenerationTokenUsage] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["system_fingerprint", "usage"]
        nullable_fields = ["system_fingerprint"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .filecitation import FileCitation, FileCitationTypedDict
from .filepath import FilePath, FilePathTypedDict
from .urlcitation import URLCitation, URLCitationTypedDict
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


OpenAIResponsesAnnotationTypedDict = TypeAliasType(
    "OpenAIResponsesAnnotationTypedDict",
    Union[FilePathTypedDict, FileCitationTypedDict, URLCitationTypedDict],
)


OpenAIResponsesAnnotation = Annotated[
    Union[
        Annotated[FileCitation, Tag("file_citation")],
        Annotated[URLCitation, Tag("url_citation")],
        Annotated[FilePath, Tag("file_path")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from openrouter.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class ChatMessageContentItemAudioInputAudioTypedDict(TypedDict):
    data: str
    format_: str


class ChatMessageContentItemAudioInputAudio(BaseModel):
    data: str

    format_: Annotated[str, pydantic.Field(alias="format")]


class ChatMessageContentItemAudioTypedDict(TypedDict):
    input_audio: ChatMessageContentItemAudioInputAudioTypedDict
    type: Literal["input_audio"]


class ChatMessageContentItemAudio(BaseModel):
    input_audio: ChatMessageContentItemAudioInputAudio

    TYPE: Annotated[
        Annotated[
            Literal["input_audio"], AfterValidator(validate_const("input_audio"))
        ],
        pydantic.Field(alias="type"),
    ] = "input_audio"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedInt
from openrouter.utils import validate_open_enum
from pydantic.functional_validators import PlainValidator
from typing import Literal, Union
from typing_extensions import Annotated, TypedDict


ChainID = Union[
    Literal[
        1,
        137,
        8453,
    ],
    UnrecognizedInt,
]


class CreateChargeRequestTypedDict(TypedDict):
    r"""Create a Coinbase charge for crypto payment"""

    amount: float
    sender: str
    chain_id: ChainID


class CreateChargeRequest(BaseModel):
    r"""Create a Coinbase charge for crypto payment"""

    amount: float

    sender: str

    chain_id: Annotated[ChainID, PlainValidator(validate_open_enum(True))]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


OpenAIResponsesRefusalContentType = Literal["refusal",]


class OpenAIResponsesRefusalContentTypedDict(TypedDict):
    type: OpenAIResponsesRefusalContentType
    refusal: str


class OpenAIResponsesRefusalContent(BaseModel):
    type: OpenAIResponsesRefusalContentType

    refusal: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.components import providername as components_providername
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import (
    FieldMetadata,
    PathParamMetadata,
    QueryParamMetadata,
    SecurityMetadata,
    validate_open_enum,
)
from pydantic.functional_validators import PlainValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


class GetParametersSecurityTypedDict(TypedDict):
    bearer: str


class GetParametersSecurity(BaseModel):
    bearer: Annotated[
        str,
        FieldMetadata(
            security=SecurityMetadata(
                scheme=True,
                scheme_type="http",
                sub_type="bearer",
                field_name="Authorization",
            )
        ),
    ]


class GetParametersRequestTypedDict(TypedDict):
    author: str
    slug: str
    provider: NotRequired[components_providername.ProviderName]


class GetParametersRequest(BaseModel):
    author: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    slug: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    provider: Annotated[
        Annotated[
            Optional[components_providername.ProviderName],
            PlainValidator(validate_open_enum(False)),
        ],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None


SupportedParameter = Union[
    Literal[
        "temperature",
        "top_p",
        "top_k",
        "min_p",
        "top_a",
        "frequency_penalty",
        "presence_penalty",
        "repetition_penalty",
        "max_tokens",
        "logit_bias",
        "logprobs",
        "top_logprobs",
        "seed",
        "response_format",
        "structured_outputs",
        "stop",
        "tools",
        "tool_choice",
        "parallel_tool_calls",
        "include_reasoning",
        "reasoning",
        "reasoning_effort",
        "web_search_options",
        "verbosity",
    ],
    UnrecognizedStr,
]


class GetParametersDataTypedDict(TypedDict):
    r"""Parameter analytics data"""

    model: str
    r"""Model identifier"""
    supported_parameters: List[SupportedParameter]
    r"""List of parameters supported by this model"""


class GetParametersData(BaseModel):
    r"""Parameter analytics data"""

    model: str
    r"""Model identifier"""

    supported_parameters: List[
        Annotated[SupportedParameter, PlainValidator(validate_open_enum(False))]
    ]
    r"""List of parameters supported by this model"""


class GetParametersResponseTypedDict(TypedDict):
    r"""Returns the parameters for the specified model"""

    data: GetParametersDataTypedDict
    r"""Parameter analytics data"""


class GetParametersResponse(BaseModel):
    r"""Returns the parameters for the specified model"""

    data: GetParametersData
    r"""Parameter analytics data"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


OpenResponsesImageGenCallCompletedType = Literal[
    "response.image_generation_call.completed",
]


class OpenResponsesImageGenCallCompletedTypedDict(TypedDict):
    r"""Image generation call completed"""

    type: OpenResponsesImageGenCallCompletedType
    item_id: str
    output_index: float
    sequence_number: float


class OpenResponsesImageGenCallCompleted(BaseModel):
    r"""Image generation call completed"""

    type: OpenResponsesImageGenCallCompletedType

    item_id: str

    output_index: float

    sequence_number: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CompletionTokensDetailsTypedDict(TypedDict):
    reasoning_tokens: NotRequired[Nullable[float]]
    audio_tokens: NotRequired[Nullable[float]]
    accepted_prediction_tokens: NotRequired[Nullable[float]]
    rejected_prediction_tokens: NotRequired[Nullable[float]]


class CompletionTokensDetails(BaseModel):
    reasoning_tokens: OptionalNullable[float] = UNSET

    audio_tokens: OptionalNullable[float] = UNSET

    accepted_prediction_tokens: OptionalNullable[float] = UNSET

    rejected_prediction_tokens: OptionalNullable[float] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "reasoning_tokens",
            "audio_tokens",
            "accepted_prediction_tokens",
            "rejected_prediction_tokens",
        ]
        nullable_fields = [
            "reasoning_tokens",
            "audio_tokens",
            "accepted_prediction_tokens",
            "rejected_prediction_tokens",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class PromptTokensDetailsTypedDict(TypedDict):
    cached_tokens: NotRequired[float]
    cache_write_tokens: NotRequired[float]
    audio_tokens: NotRequired[float]
    video_tokens: NotRequired[float]


class PromptTokensDetails(BaseModel):
    cached_tokens: Optional[float] = None

    cache_write_tokens: Optional[float] = None

    audio_tokens: Optional[float] = None

    video_tokens: Optional[float] = None


class ChatGenerationTokenUsageTypedDict(TypedDict):
    completion_tokens: float
    prompt_tokens: float
    total_tokens: float
    completion_tokens_details: NotRequired[Nullable[CompletionTokensDetailsTypedDict]]
    prompt_tokens_details: NotRequired[Nullable[PromptTokensDetailsTypedDict]]


class ChatGenerationTokenUsage(BaseModel):
    completion_tokens: float

    prompt_tokens: float

    total_tokens: float

    completion_tokens_details: OptionalNullable[CompletionTokensDetails] = UNSET

    prompt_tokens_details: OptionalNullable[PromptTokensDetails] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["completion_tokens_details", "prompt_tokens_details"]
        nullable_fields = ["completion_tokens_details", "prompt_tokens_details"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .percentilethroughputcutoffs import (
    PercentileThroughputCutoffs,
    PercentileThroughputCutoffsTypedDict,
)
from typing import Any, Union
from typing_extensions import TypeAliasType


PreferredMinThroughputTypedDict = TypeAliasType(
    "PreferredMinThroughputTypedDict",
    Union[PercentileThroughputCutoffsTypedDict, float, Any],
)
r"""Preferred minimum throughput (in tokens per second). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints below the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""


PreferredMinThroughput = TypeAliasType(
    "PreferredMinThroughput", Union[PercentileThroughputCutoffs, float, Any]
)
r"""Preferred minimum throughput (in tokens per second). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints below the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.components import providerpreferences as components_providerpreferences
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import get_discriminator, validate_open_enum
from pydantic import Discriminator, Tag
from pydantic.functional_validators import PlainValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


TypeImageURL = Literal["image_url",]


class ImageURLTypedDict(TypedDict):
    url: str


class ImageURL(BaseModel):
    url: str


class ContentImageURLTypedDict(TypedDict):
    type: TypeImageURL
    image_url: ImageURLTypedDict


class ContentImageURL(BaseModel):
    type: TypeImageURL

    image_url: ImageURL


TypeText = Literal["text",]


class ContentTextTypedDict(TypedDict):
    type: TypeText
    text: str


class ContentText(BaseModel):
    type: TypeText

    text: str


ContentTypedDict = TypeAliasType(
    "ContentTypedDict", Union[ContentTextTypedDict, ContentImageURLTypedDict]
)


Content = Annotated[
    Union[
        Annotated[ContentText, Tag("text")],
        Annotated[ContentImageURL, Tag("image_url")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class InputTypedDict(TypedDict):
    content: List[ContentTypedDict]


class Input(BaseModel):
    content: List[Content]


InputUnionTypedDict = TypeAliasType(
    "InputUnionTypedDict",
    Union[str, List[str], List[float], List[List[float]], List[InputTypedDict]],
)


InputUnion = TypeAliasType(
    "InputUnion", Union[str, List[str], List[float], List[List[float]], List[Input]]
)


EncodingFormat = Union[
    Literal[
        "float",
        "base64",
    ],
    UnrecognizedStr,
]


class CreateEmbeddingsRequestTypedDict(TypedDict):
    input: InputUnionTypedDict
    model: str
    encoding_format: NotRequired[EncodingFormat]
    dimensions: NotRequired[int]
    user: NotRequired[str]
    provider: NotRequired[components_providerpreferences.ProviderPreferencesTypedDict]
    r"""Provider routing preferences for the request."""
    input_type: NotRequired[str]


class CreateEmbeddingsRequest(BaseModel):
    input: InputUnion

    model: str

    encoding_format: Annotated[
        Optional[EncodingFormat], PlainValidator(validate_open_enum(False))
    ] = None

    dimensions: Optional[int] = None

    user: Optional[str] = None

    provider: Optional[components_providerpreferences.ProviderPreferences] = None
    r"""Provider routing preferences for the request."""

    input_type: Optional[str] = None


Object = Literal["list",]


ObjectEmbedding = Literal["embedding",]


EmbeddingTypedDict = TypeAliasType("EmbeddingTypedDict", Union[List[float], str])


Embedding = TypeAliasType("Embedding", Union[List[float], str])


class CreateEmbeddingsDataTypedDict(TypedDict):
    object: ObjectEmbedding
    embedding: EmbeddingTypedDict
    index: NotRequired[float]


class CreateEmbeddingsData(BaseModel):
    object: ObjectEmbedding

    embedding: Embedding

    index: Optional[float] = None


class UsageTypedDict(TypedDict):
    prompt_tokens: float
    total_tokens: float
    cost: NotRequired[float]


class Usage(BaseModel):
    prompt_tokens: float

    total_tokens: float

    cost: Optional[float] = None


class CreateEmbeddingsResponseBodyTypedDict(TypedDict):
    r"""Embedding response"""

    object: Object
    data: List[CreateEmbeddingsDataTypedDict]
    model: str
    id: NotRequired[str]
    usage: NotRequired[UsageTypedDict]


class CreateEmbeddingsResponseBody(BaseModel):
    r"""Embedding response"""

    object: Object

    data: List[CreateEmbeddingsData]

    model: str

    id: Optional[str] = None

    usage: Optional[Usage] = None


CreateEmbeddingsResponseTypedDict = TypeAliasType(
    "CreateEmbeddingsResponseTypedDict",
    Union[CreateEmbeddingsResponseBodyTypedDict, str],
)


CreateEmbeddingsResponse = TypeAliasType(
    "CreateEmbeddingsResponse", Union[CreateEmbeddingsResponseBody, str]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import UnrecognizedStr
from typing import Literal, Union


WebSearchEngine = Union[
    Literal[
        "native",
        "exa",
    ],
    UnrecognizedStr,
]
r"""The search engine to use for web search."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from openrouter.components import (
    internalserverresponseerrordata as components_internalserverresponseerrordata,
)
from openrouter.errors import OpenRouterError
from openrouter.types import BaseModel, OptionalNullable, UNSET
from typing import Optional


class InternalServerResponseErrorData(BaseModel):
    error: components_internalserverresponseerrordata.InternalServerResponseErrorData
    r"""Error data for InternalServerResponse"""

    user_id: OptionalNullable[str] = UNSET


@dataclass(unsafe_hash=True)
class InternalServerResponseError(OpenRouterError):
    r"""Internal Server Error - Unexpected server error"""

    data: InternalServerResponseErrorData = field(hash=False)

    def __init__(
        self,
        data: InternalServerResponseErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.error.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .percentilelatencycutoffs import (
    PercentileLatencyCutoffs,
    PercentileLatencyCutoffsTypedDict,
)
from typing import Any, Union
from typing_extensions import TypeAliasType


PreferredMaxLatencyTypedDict = TypeAliasType(
    "PreferredMaxLatencyTypedDict", Union[PercentileLatencyCutoffsTypedDict, float, Any]
)
r"""Preferred maximum latency (in seconds). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints above the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""


PreferredMaxLatency = TypeAliasType(
    "PreferredMaxLatency", Union[PercentileLatencyCutoffs, float, Any]
)
r"""Preferred maximum latency (in seconds). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints above the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import (
    FieldMetadata,
    PathParamMetadata,
    RequestMetadata,
    validate_open_enum,
)
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


UpdateKeysLimitReset = Union[
    Literal[
        "daily",
        "weekly",
        "monthly",
    ],
    UnrecognizedStr,
]
r"""New limit reset type for the API key (daily, weekly, monthly, or null for no reset). Resets happen automatically at midnight UTC, and weeks are Monday through Sunday."""


class UpdateKeysRequestBodyTypedDict(TypedDict):
    name: NotRequired[str]
    r"""New name for the API key"""
    disabled: NotRequired[bool]
    r"""Whether to disable the API key"""
    limit: NotRequired[Nullable[float]]
    r"""New spending limit for the API key in USD"""
    limit_reset: NotRequired[Nullable[UpdateKeysLimitReset]]
    r"""New limit reset type for the API key (daily, weekly, monthly, or null for no reset). Resets happen automatically at midnight UTC, and weeks are Monday through Sunday."""
    include_byok_in_limit: NotRequired[bool]
    r"""Whether to include BYOK usage in the limit"""


class UpdateKeysRequestBody(BaseModel):
    name: Optional[str] = None
    r"""New name for the API key"""

    disabled: Optional[bool] = None
    r"""Whether to disable the API key"""

    limit: OptionalNullable[float] = UNSET
    r"""New spending limit for the API key in USD"""

    limit_reset: Annotated[
        OptionalNullable[UpdateKeysLimitReset],
        PlainValidator(validate_open_enum(False)),
    ] = UNSET
    r"""New limit reset type for the API key (daily, weekly, monthly, or null for no reset). Resets happen automatically at midnight UTC, and weeks are Monday through Sunday."""

    include_byok_in_limit: Optional[bool] = None
    r"""Whether to include BYOK usage in the limit"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "name",
            "disabled",
            "limit",
            "limit_reset",
            "include_byok_in_limit",
        ]
        nullable_fields = ["limit", "limit_reset"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class UpdateKeysRequestTypedDict(TypedDict):
    hash: str
    r"""The hash identifier of the API key to update"""
    request_body: UpdateKeysRequestBodyTypedDict


class UpdateKeysRequest(BaseModel):
    hash: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The hash identifier of the API key to update"""

    request_body: Annotated[
        UpdateKeysRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class UpdateKeysDataTypedDict(TypedDict):
    r"""The updated API key information"""

    hash: str
    r"""Unique hash identifier for the API key"""
    name: str
    r"""Name of the API key"""
    label: str
    r"""Human-readable label for the API key"""
    disabled: bool
    r"""Whether the API key is disabled"""
    limit: Nullable[float]
    r"""Spending limit for the API key in USD"""
    limit_remaining: Nullable[float]
    r"""Remaining spending limit in USD"""
    limit_reset: Nullable[str]
    r"""Type of limit reset for the API key"""
    include_byok_in_limit: bool
    r"""Whether to include external BYOK usage in the credit limit"""
    usage: float
    r"""Total OpenRouter credit usage (in USD) for the API key"""
    usage_daily: float
    r"""OpenRouter credit usage (in USD) for the current UTC day"""
    usage_weekly: float
    r"""OpenRouter credit usage (in USD) for the current UTC week (Monday-Sunday)"""
    usage_monthly: float
    r"""OpenRouter credit usage (in USD) for the current UTC month"""
    byok_usage: float
    r"""Total external BYOK usage (in USD) for the API key"""
    byok_usage_daily: float
    r"""External BYOK usage (in USD) for the current UTC day"""
    byok_usage_weekly: float
    r"""External BYOK usage (in USD) for the current UTC week (Monday-Sunday)"""
    byok_usage_monthly: float
    r"""External BYOK usage (in USD) for current UTC month"""
    created_at: str
    r"""ISO 8601 timestamp of when the API key was created"""
    updated_at: Nullable[str]
    r"""ISO 8601 timestamp of when the API key was last updated"""
    expires_at: NotRequired[Nullable[datetime]]
    r"""ISO 8601 UTC timestamp when the API key expires, or null if no expiration"""


class UpdateKeysData(BaseModel):
    r"""The updated API key information"""

    hash: str
    r"""Unique hash identifier for the API key"""

    name: str
    r"""Name of the API key"""

    label: str
    r"""Human-readable label for the API key"""

    disabled: bool
    r"""Whether the API key is disabled"""

    limit: Nullable[float]
    r"""Spending limit for the API key in USD"""

    limit_remaining: Nullable[float]
    r"""Remaining spending limit in USD"""

    limit_reset: Nullable[str]
    r"""Type of limit reset for the API key"""

    include_byok_in_limit: bool
    r"""Whether to include external BYOK usage in the credit limit"""

    usage: float
    r"""Total OpenRouter credit usage (in USD) for the API key"""

    usage_daily: float
    r"""OpenRouter credit usage (in USD) for the current UTC day"""

    usage_weekly: float
    r"""OpenRouter credit usage (in USD) for the current UTC week (Monday-Sunday)"""

    usage_monthly: float
    r"""OpenRouter credit usage (in USD) for the current UTC month"""

    byok_usage: float
    r"""Total external BYOK usage (in USD) for the API key"""

    byok_usage_daily: float
    r"""External BYOK usage (in USD) for the current UTC day"""

    byok_usage_weekly: float
    r"""External BYOK usage (in USD) for the current UTC week (Monday-Sunday)"""

    byok_usage_monthly: float
    r"""External BYOK usage (in USD) for current UTC month"""

    created_at: str
    r"""ISO 8601 timestamp of when the API key was created"""

    updated_at: Nullable[str]
    r"""ISO 8601 timestamp of when the API key was last updated"""

    expires_at: OptionalNullable[datetime] = UNSET
    r"""ISO 8601 UTC timestamp when the API key expires, or null if no expiration"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["expires_at"]
        nullable_fields = [
            "limit",
            "limit_remaining",
            "limit_reset",
            "updated_at",
            "expires_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class UpdateKeysResponseTypedDict(TypedDict):
    r"""API key updated successfully"""

    data: UpdateKeysDataTypedDict
    r"""The updated API key information"""


class UpdateKeysResponse(BaseModel):
    r"""API key updated successfully"""

    data: UpdateKeysData
    r"""The updated API key information"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class InputTokensDetailsTypedDict(TypedDict):
    cached_tokens: float


class InputTokensDetails(BaseModel):
    cached_tokens: float


class OutputTokensDetailsTypedDict(TypedDict):
    reasoning_tokens: float


class OutputTokensDetails(BaseModel):
    reasoning_tokens: float


class CostDetailsTypedDict(TypedDict):
    upstream_inference_input_cost: float
    upstream_inference_output_cost: float
    upstream_inference_cost: NotRequired[Nullable[float]]


class CostDetails(BaseModel):
    upstream_inference_input_cost: float

    upstream_inference_output_cost: float

    upstream_inference_cost: OptionalNullable[float] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["upstream_inference_cost"]
        nullable_fields = ["upstream_inference_cost"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class OpenResponsesUsageTypedDict(TypedDict):
    r"""Token usage information for the response"""

    input_tokens: float
    input_tokens_details: InputTokensDetailsTypedDict
    output_tokens: float
    output_tokens_details: OutputTokensDetailsTypedDict
    total_tokens: float
    cost: NotRequired[Nullable[float]]
    r"""Cost of the completion"""
    is_byok: NotRequired[bool]
    r"""Whether a request was made using a Bring Your Own Key configuration"""
    cost_details: NotRequired[CostDetailsTypedDict]


class OpenResponsesUsage(BaseModel):
    r"""Token usage information for the response"""

    input_tokens: float

    input_tokens_details: InputTokensDetails

    output_tokens: float

    output_tokens_details: OutputTokensDetails

    total_tokens: float

    cost: OptionalNullable[float] = UNSET
    r"""Cost of the completion"""

    is_byok: Optional[bool] = None
    r"""Whether a request was made using a Bring Your Own Key configuration"""

    cost_details: Optional[CostDetails] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["cost", "is_byok", "cost_details"]
        nullable_fields = ["cost"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.components import activityitem as components_activityitem
from openrouter.types import BaseModel
from openrouter.utils import FieldMetadata, QueryParamMetadata
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GetUserActivityRequestTypedDict(TypedDict):
    date_: NotRequired[str]
    r"""Filter by a single UTC date in the last 30 days (YYYY-MM-DD format)."""


class GetUserActivityRequest(BaseModel):
    date_: Annotated[
        Optional[str],
        pydantic.Field(alias="date"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter by a single UTC date in the last 30 days (YYYY-MM-DD format)."""


class GetUserActivityResponseTypedDict(TypedDict):
    r"""Returns user activity data grouped by endpoint"""

    data: List[components_activityitem.ActivityItemTypedDict]
    r"""List of activity items"""


class GetUserActivityResponse(BaseModel):
    r"""Returns user activity data grouped by endpoint"""

    data: List[components_activityitem.ActivityItem]
    r"""List of activity items"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .openairesponsesrefusalcontent import (
    OpenAIResponsesRefusalContent,
    OpenAIResponsesRefusalContentTypedDict,
)
from .responseoutputtext import ResponseOutputText, ResponseOutputTextTypedDict
from openrouter.types import BaseModel
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


OutputMessageRole = Literal["assistant",]


OutputMessageType = Literal["message",]


OutputMessageStatusInProgress = Literal["in_progress",]


OutputMessageStatusIncomplete = Literal["incomplete",]


OutputMessageStatusCompleted = Literal["completed",]


OutputMessageStatusUnionTypedDict = TypeAliasType(
    "OutputMessageStatusUnionTypedDict",
    Union[
        OutputMessageStatusCompleted,
        OutputMessageStatusIncomplete,
        OutputMessageStatusInProgress,
    ],
)


OutputMessageStatusUnion = TypeAliasType(
    "OutputMessageStatusUnion",
    Union[
        OutputMessageStatusCompleted,
        OutputMessageStatusIncomplete,
        OutputMessageStatusInProgress,
    ],
)


OutputMessageContentTypedDict = TypeAliasType(
    "OutputMessageContentTypedDict",
    Union[OpenAIResponsesRefusalContentTypedDict, ResponseOutputTextTypedDict],
)


OutputMessageContent = Annotated[
    Union[
        Annotated[ResponseOutputText, Tag("output_text")],
        Annotated[OpenAIResponsesRefusalContent, Tag("refusal")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class OutputMessageTypedDict(TypedDict):
    id: str
    role: OutputMessageRole
    type: OutputMessageType
    content: List[OutputMessageContentTypedDict]
    status: NotRequired[OutputMessageStatusUnionTypedDict]


class OutputMessage(BaseModel):
    id: str

    role: OutputMessageRole

    type: OutputMessageType

    content: List[OutputMessageContent]

    status: Optional[OutputMessageStatusUnion] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .completionchoice import CompletionChoice, CompletionChoiceTypedDict
from .completionusage import CompletionUsage, CompletionUsageTypedDict
from openrouter.types import BaseModel
from openrouter.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CompletionResponseTypedDict(TypedDict):
    id: str
    created: float
    model: str
    choices: List[CompletionChoiceTypedDict]
    object: Literal["text_completion"]
    provider: NotRequired[str]
    system_fingerprint: NotRequired[str]
    usage: NotRequired[CompletionUsageTypedDict]


class CompletionResponse(BaseModel):
    id: str

    created: float

    model: str

    choices: List[CompletionChoice]

    OBJECT: Annotated[
        Annotated[
            Literal["text_completion"],
            AfterValidator(validate_const("text_completion")),
        ],
        pydantic.Field(alias="object"),
    ] = "text_completion"

    provider: Optional[str] = None

    system_fingerprint: Optional[str] = None

    usage: Optional[CompletionUsage] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .datacollection import DataCollection
from .preferredmaxlatency import PreferredMaxLatency, PreferredMaxLatencyTypedDict
from .preferredminthroughput import (
    PreferredMinThroughput,
    PreferredMinThroughputTypedDict,
)
from .providername import ProviderName
from .providersort import ProviderSort
from .quantization import Quantization
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ProviderPreferencesOrderTypedDict = TypeAliasType(
    "ProviderPreferencesOrderTypedDict", Union[ProviderName, str]
)


ProviderPreferencesOrder = TypeAliasType(
    "ProviderPreferencesOrder",
    Union[Annotated[ProviderName, PlainValidator(validate_open_enum(False))], str],
)


ProviderPreferencesOnlyTypedDict = TypeAliasType(
    "ProviderPreferencesOnlyTypedDict", Union[ProviderName, str]
)


ProviderPreferencesOnly = TypeAliasType(
    "ProviderPreferencesOnly",
    Union[Annotated[ProviderName, PlainValidator(validate_open_enum(False))], str],
)


ProviderPreferencesIgnoreTypedDict = TypeAliasType(
    "ProviderPreferencesIgnoreTypedDict", Union[ProviderName, str]
)


ProviderPreferencesIgnore = TypeAliasType(
    "ProviderPreferencesIgnore",
    Union[Annotated[ProviderName, PlainValidator(validate_open_enum(False))], str],
)


SortEnum = Union[
    Literal[
        "price",
        "throughput",
        "latency",
    ],
    UnrecognizedStr,
]


ProviderSortConfigEnum = Literal[
    "price",
    "throughput",
    "latency",
]


ProviderPreferencesPartition = Union[
    Literal[
        "model",
        "none",
    ],
    UnrecognizedStr,
]


class ProviderPreferencesProviderSortConfigTypedDict(TypedDict):
    by: NotRequired[Nullable[ProviderSort]]
    partition: NotRequired[Nullable[ProviderPreferencesPartition]]


class ProviderPreferencesProviderSortConfig(BaseModel):
    by: Annotated[
        OptionalNullable[ProviderSort], PlainValidator(validate_open_enum(False))
    ] = UNSET

    partition: Annotated[
        OptionalNullable[ProviderPreferencesPartition],
        PlainValidator(validate_open_enum(False)),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["by", "partition"]
        nullable_fields = ["by", "partition"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


ProviderSortConfigUnionTypedDict = TypeAliasType(
    "ProviderSortConfigUnionTypedDict",
    Union[ProviderPreferencesProviderSortConfigTypedDict, ProviderSortConfigEnum],
)


ProviderSortConfigUnion = TypeAliasType(
    "ProviderSortConfigUnion",
    Union[ProviderPreferencesProviderSortConfig, ProviderSortConfigEnum],
)


ProviderPreferencesProviderSort = Union[
    Literal[
        "price",
        "throughput",
        "latency",
    ],
    UnrecognizedStr,
]


ProviderPreferencesSortUnionTypedDict = TypeAliasType(
    "ProviderPreferencesSortUnionTypedDict",
    Union[ProviderPreferencesProviderSort, ProviderSortConfigUnionTypedDict, SortEnum],
)
r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""


ProviderPreferencesSortUnion = TypeAliasType(
    "ProviderPreferencesSortUnion",
    Union[
        Annotated[
            ProviderPreferencesProviderSort, PlainValidator(validate_open_enum(False))
        ],
        ProviderSortConfigUnion,
        Annotated[SortEnum, PlainValidator(validate_open_enum(False))],
    ],
)
r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""


class ProviderPreferencesMaxPriceTypedDict(TypedDict):
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    prompt: NotRequired[str]
    r"""A value in string format that is a large number"""
    completion: NotRequired[str]
    r"""A value in string format that is a large number"""
    image: NotRequired[str]
    r"""A value in string format that is a large number"""
    audio: NotRequired[str]
    r"""A value in string format that is a large number"""
    request: NotRequired[str]
    r"""A value in string format that is a large number"""


class ProviderPreferencesMaxPrice(BaseModel):
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    prompt: Optional[str] = None
    r"""A value in string format that is a large number"""

    completion: Optional[str] = None
    r"""A value in string format that is a large number"""

    image: Optional[str] = None
    r"""A value in string format that is a large number"""

    audio: Optional[str] = None
    r"""A value in string format that is a large number"""

    request: Optional[str] = None
    r"""A value in string format that is a large number"""


class ProviderPreferencesTypedDict(TypedDict):
    r"""Provider routing preferences for the request."""

    allow_fallbacks: NotRequired[Nullable[bool]]
    r"""Whether to allow backup providers to serve requests
    - true: (default) when the primary provider (or your custom providers in \"order\") is unavailable, use the next best provider.
    - false: use only the primary/custom provider, and return the upstream error if it's unavailable.

    """
    require_parameters: NotRequired[Nullable[bool]]
    r"""Whether to filter providers to only those that support the parameters you've provided. If this setting is omitted or set to false, then providers will receive only the parameters they support, and ignore the rest."""
    data_collection: NotRequired[Nullable[DataCollection]]
    r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
    - allow: (default) allow providers which store user data non-transiently and may train on it

    - deny: use only providers which do not collect user data.
    """
    zdr: NotRequired[Nullable[bool]]
    r"""Whether to restrict routing to only ZDR (Zero Data Retention) endpoints. When true, only endpoints that do not retain prompts will be used."""
    enforce_distillable_text: NotRequired[Nullable[bool]]
    r"""Whether to restrict routing to only models that allow text distillation. When true, only models where the author has allowed distillation will be used."""
    order: NotRequired[Nullable[List[ProviderPreferencesOrderTypedDict]]]
    r"""An ordered list of provider slugs. The router will attempt to use the first provider in the subset of this list that supports your requested model, and fall back to the next if it is unavailable. If no providers are available, the request will fail with an error message."""
    only: NotRequired[Nullable[List[ProviderPreferencesOnlyTypedDict]]]
    r"""List of provider slugs to allow. If provided, this list is merged with your account-wide allowed provider settings for this request."""
    ignore: NotRequired[Nullable[List[ProviderPreferencesIgnoreTypedDict]]]
    r"""List of provider slugs to ignore. If provided, this list is merged with your account-wide ignored provider settings for this request."""
    quantizations: NotRequired[Nullable[List[Quantization]]]
    r"""A list of quantization levels to filter the provider by."""
    sort: NotRequired[Nullable[ProviderPreferencesSortUnionTypedDict]]
    max_price: NotRequired[ProviderPreferencesMaxPriceTypedDict]
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""
    preferred_min_throughput: NotRequired[Nullable[PreferredMinThroughputTypedDict]]
    r"""Preferred minimum throughput (in tokens per second). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints below the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""
    preferred_max_latency: NotRequired[Nullable[PreferredMaxLatencyTypedDict]]
    r"""Preferred maximum latency (in seconds). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints above the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""


class ProviderPreferences(BaseModel):
    r"""Provider routing preferences for the request."""

    allow_fallbacks: OptionalNullable[bool] = UNSET
    r"""Whether to allow backup providers to serve requests
    - true: (default) when the primary provider (or your custom providers in \"order\") is unavailable, use the next best provider.
    - false: use only the primary/custom provider, and return the upstream error if it's unavailable.

    """

    require_parameters: OptionalNullable[bool] = UNSET
    r"""Whether to filter providers to only those that support the parameters you've provided. If this setting is omitted or set to false, then providers will receive only the parameters they support, and ignore the rest."""

    data_collection: Annotated[
        OptionalNullable[DataCollection], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
    - allow: (default) allow providers which store user data non-transiently and may train on it

    - deny: use only providers which do not collect user data.
    """

    zdr: OptionalNullable[bool] = UNSET
    r"""Whether to restrict routing to only ZDR (Zero Data Retention) endpoints. When true, only endpoints that do not retain prompts will be used."""

    enforce_distillable_text: OptionalNullable[bool] = UNSET
    r"""Whether to restrict routing to only models that allow text distillation. When true, only models where the author has allowed distillation will be used."""

    order: OptionalNullable[List[ProviderPreferencesOrder]] = UNSET
    r"""An ordered list of provider slugs. The router will attempt to use the first provider in the subset of this list that supports your requested model, and fall back to the next if it is unavailable. If no providers are available, the request will fail with an error message."""

    only: OptionalNullable[List[ProviderPreferencesOnly]] = UNSET
    r"""List of provider slugs to allow. If provided, this list is merged with your account-wide allowed provider settings for this request."""

    ignore: OptionalNullable[List[ProviderPreferencesIgnore]] = UNSET
    r"""List of provider slugs to ignore. If provided, this list is merged with your account-wide ignored provider settings for this request."""

    quantizations: OptionalNullable[
        List[Annotated[Quantization, PlainValidator(validate_open_enum(False))]]
    ] = UNSET
    r"""A list of quantization levels to filter the provider by."""

    sort: OptionalNullable[ProviderPreferencesSortUnion] = UNSET

    max_price: Optional[ProviderPreferencesMaxPrice] = None
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    preferred_min_throughput: OptionalNullable[PreferredMinThroughput] = UNSET
    r"""Preferred minimum throughput (in tokens per second). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints below the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""

    preferred_max_latency: OptionalNullable[PreferredMaxLatency] = UNSET
    r"""Preferred maximum latency (in seconds). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints above the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "allow_fallbacks",
            "require_parameters",
            "data_collection",
            "zdr",
            "enforce_distillable_text",
            "order",
            "only",
            "ignore",
            "quantizations",
            "sort",
            "max_price",
            "preferred_min_throughput",
            "preferred_max_latency",
        ]
        nullable_fields = [
            "allow_fallbacks",
            "require_parameters",
            "data_collection",
            "zdr",
            "enforce_distillable_text",
            "order",
            "only",
            "ignore",
            "quantizations",
            "sort",
            "preferred_min_throughput",
            "preferred_max_latency",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .model import Model, ModelTypedDict
from openrouter.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ModelsListResponseTypedDict(TypedDict):
    r"""List of available models"""

    data: List[ModelTypedDict]
    r"""List of available models"""


class ModelsListResponse(BaseModel):
    r"""List of available models"""

    data: List[Model]
    r"""List of available models"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from enum import Enum
from openrouter import components, errors, operations, utils
from openrouter._hooks import HookContext
from openrouter.types import OptionalNullable, UNSET
from openrouter.utils import eventstreaming, get_security_from_env
from openrouter.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Literal, Mapping, Optional, Union, overload


class SendAcceptEnum(str, Enum):
    APPLICATION_JSON = "application/json"
    TEXT_EVENT_STREAM = "text/event-stream"


class Chat(BaseSDK):
    @overload
    def send(
        self,
        *,
        messages: Union[List[components.Message], List[components.MessageTypedDict]],
        provider: OptionalNullable[
            Union[
                components.ChatGenerationParamsProvider,
                components.ChatGenerationParamsProviderTypedDict,
            ]
        ] = UNSET,
        plugins: Optional[
            Union[
                List[components.ChatGenerationParamsPluginUnion],
                List[components.ChatGenerationParamsPluginUnionTypedDict],
            ]
        ] = None,
        route: OptionalNullable[components.Route] = UNSET,
        user: Optional[str] = None,
        session_id: Optional[str] = None,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[Dict[str, float]] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        top_logprobs: OptionalNullable[float] = UNSET,
        max_completion_tokens: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[float] = UNSET,
        metadata: Optional[Dict[str, str]] = None,
        presence_penalty: OptionalNullable[float] = UNSET,
        reasoning: Optional[
            Union[components.Reasoning, components.ReasoningTypedDict]
        ] = None,
        response_format: Optional[
            Union[
                components.ChatGenerationParamsResponseFormatUnion,
                components.ChatGenerationParamsResponseFormatUnionTypedDict,
            ]
        ] = None,
        seed: OptionalNullable[int] = UNSET,
        stop: OptionalNullable[
            Union[
                components.ChatGenerationParamsStop,
                components.ChatGenerationParamsStopTypedDict,
            ]
        ] = UNSET,
        stream: Union[Literal[False], None] = None,
        stream_options: OptionalNullable[
            Union[components.ChatStreamOptions, components.ChatStreamOptionsTypedDict]
        ] = UNSET,
        temperature: OptionalNullable[float] = UNSET,
        tool_choice: Optional[Any] = None,
        tools: Optional[
            Union[
                List[components.ToolDefinitionJSON],
                List[components.ToolDefinitionJSONTypedDict],
            ]
        ] = None,
        top_p: OptionalNullable[float] = UNSET,
        debug: Optional[Union[components.Debug, components.DebugTypedDict]] = None,
        image_config: Optional[
            Union[
                Dict[str, components.ChatGenerationParamsImageConfig],
                Dict[str, components.ChatGenerationParamsImageConfigTypedDict],
            ]
        ] = None,
        modalities: Optional[List[components.Modality]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> components.ChatResponse:
        r"""Create a chat completion

        Sends a request for a model response for the given chat conversation. Supports both streaming and non-streaming modes.

        :param messages:
        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param route:
        :param user:
        :param session_id: A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 128 characters.
        :param model:
        :param models:
        :param frequency_penalty:
        :param logit_bias:
        :param logprobs:
        :param top_logprobs:
        :param max_completion_tokens:
        :param max_tokens:
        :param metadata:
        :param presence_penalty:
        :param reasoning:
        :param response_format:
        :param seed:
        :param stop:
        :param stream:
        :param stream_options:
        :param temperature:
        :param tool_choice:
        :param tools:
        :param top_p:
        :param debug:
        :param image_config:
        :param modalities:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    @overload
    def send(
        self,
        *,
        messages: Union[List[components.Message], List[components.MessageTypedDict]],
        provider: OptionalNullable[
            Union[
                components.ChatGenerationParamsProvider,
                components.ChatGenerationParamsProviderTypedDict,
            ]
        ] = UNSET,
        plugins: Optional[
            Union[
                List[components.ChatGenerationParamsPluginUnion],
                List[components.ChatGenerationParamsPluginUnionTypedDict],
            ]
        ] = None,
        route: OptionalNullable[components.Route] = UNSET,
        user: Optional[str] = None,
        session_id: Optional[str] = None,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[Dict[str, float]] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        top_logprobs: OptionalNullable[float] = UNSET,
        max_completion_tokens: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[float] = UNSET,
        metadata: Optional[Dict[str, str]] = None,
        presence_penalty: OptionalNullable[float] = UNSET,
        reasoning: Optional[
            Union[components.Reasoning, components.ReasoningTypedDict]
        ] = None,
        response_format: Optional[
            Union[
                components.ChatGenerationParamsResponseFormatUnion,
                components.ChatGenerationParamsResponseFormatUnionTypedDict,
            ]
        ] = None,
        seed: OptionalNullable[int] = UNSET,
        stop: OptionalNullable[
            Union[
                components.ChatGenerationParamsStop,
                components.ChatGenerationParamsStopTypedDict,
            ]
        ] = UNSET,
        stream: Literal[True],
        stream_options: OptionalNullable[
            Union[components.ChatStreamOptions, components.ChatStreamOptionsTypedDict]
        ] = UNSET,
        temperature: OptionalNullable[float] = UNSET,
        tool_choice: Optional[Any] = None,
        tools: Optional[
            Union[
                List[components.ToolDefinitionJSON],
                List[components.ToolDefinitionJSONTypedDict],
            ]
        ] = None,
        top_p: OptionalNullable[float] = UNSET,
        debug: Optional[Union[components.Debug, components.DebugTypedDict]] = None,
        image_config: Optional[
            Union[
                Dict[str, components.ChatGenerationParamsImageConfig],
                Dict[str, components.ChatGenerationParamsImageConfigTypedDict],
            ]
        ] = None,
        modalities: Optional[List[components.Modality]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStream[components.ChatStreamingResponseChunkData]:
        r"""Create a chat completion

        Sends a request for a model response for the given chat conversation. Supports both streaming and non-streaming modes.

        :param messages:
        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param route:
        :param user:
        :param session_id: A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 128 characters.
        :param model:
        :param models:
        :param frequency_penalty:
        :param logit_bias:
        :param logprobs:
        :param top_logprobs:
        :param max_completion_tokens:
        :param max_tokens:
        :param metadata:
        :param presence_penalty:
        :param reasoning:
        :param response_format:
        :param seed:
        :param stop:
        :param stream:
        :param stream_options:
        :param temperature:
        :param tool_choice:
        :param tools:
        :param top_p:
        :param debug:
        :param image_config:
        :param modalities:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    def send(
        self,
        *,
        messages: Union[List[components.Message], List[components.MessageTypedDict]],
        provider: OptionalNullable[
            Union[
                components.ChatGenerationParamsProvider,
                components.ChatGenerationParamsProviderTypedDict,
            ]
        ] = UNSET,
        plugins: Optional[
            Union[
                List[components.ChatGenerationParamsPluginUnion],
                List[components.ChatGenerationParamsPluginUnionTypedDict],
            ]
        ] = None,
        route: OptionalNullable[components.Route] = UNSET,
        user: Optional[str] = None,
        session_id: Optional[str] = None,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[Dict[str, float]] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        top_logprobs: OptionalNullable[float] = UNSET,
        max_completion_tokens: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[float] = UNSET,
        metadata: Optional[Dict[str, str]] = None,
        presence_penalty: OptionalNullable[float] = UNSET,
        reasoning: Optional[
            Union[components.Reasoning, components.ReasoningTypedDict]
        ] = None,
        response_format: Optional[
            Union[
                components.ChatGenerationParamsResponseFormatUnion,
                components.ChatGenerationParamsResponseFormatUnionTypedDict,
            ]
        ] = None,
        seed: OptionalNullable[int] = UNSET,
        stop: OptionalNullable[
            Union[
                components.ChatGenerationParamsStop,
                components.ChatGenerationParamsStopTypedDict,
            ]
        ] = UNSET,
        stream: Optional[bool] = False,
        stream_options: OptionalNullable[
            Union[components.ChatStreamOptions, components.ChatStreamOptionsTypedDict]
        ] = UNSET,
        temperature: OptionalNullable[float] = UNSET,
        tool_choice: Optional[Any] = None,
        tools: Optional[
            Union[
                List[components.ToolDefinitionJSON],
                List[components.ToolDefinitionJSONTypedDict],
            ]
        ] = None,
        top_p: OptionalNullable[float] = UNSET,
        debug: Optional[Union[components.Debug, components.DebugTypedDict]] = None,
        image_config: Optional[
            Union[
                Dict[str, components.ChatGenerationParamsImageConfig],
                Dict[str, components.ChatGenerationParamsImageConfigTypedDict],
            ]
        ] = None,
        modalities: Optional[List[components.Modality]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.SendChatCompletionRequestResponse:
        r"""Create a chat completion

        Sends a request for a model response for the given chat conversation. Supports both streaming and non-streaming modes.

        :param messages:
        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param route:
        :param user:
        :param session_id: A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 128 characters.
        :param model:
        :param models:
        :param frequency_penalty:
        :param logit_bias:
        :param logprobs:
        :param top_logprobs:
        :param max_completion_tokens:
        :param max_tokens:
        :param metadata:
        :param presence_penalty:
        :param reasoning:
        :param response_format:
        :param seed:
        :param stop:
        :param stream:
        :param stream_options:
        :param temperature:
        :param tool_choice:
        :param tools:
        :param top_p:
        :param debug:
        :param image_config:
        :param modalities:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = components.ChatGenerationParams(
            provider=utils.get_pydantic_model(
                provider, OptionalNullable[components.ChatGenerationParamsProvider]
            ),
            plugins=utils.get_pydantic_model(
                plugins, Optional[List[components.ChatGenerationParamsPluginUnion]]
            ),
            route=route,
            user=user,
            session_id=session_id,
            messages=utils.get_pydantic_model(messages, List[components.Message]),
            model=model,
            models=models,
            frequency_penalty=frequency_penalty,
            logit_bias=logit_bias,
            logprobs=logprobs,
            top_logprobs=top_logprobs,
            max_completion_tokens=max_completion_tokens,
            max_tokens=max_tokens,
            metadata=metadata,
            presence_penalty=presence_penalty,
            reasoning=utils.get_pydantic_model(
                reasoning, Optional[components.Reasoning]
            ),
            response_format=utils.get_pydantic_model(
                response_format,
                Optional[components.ChatGenerationParamsResponseFormatUnion],
            ),
            seed=seed,
            stop=stop,
            stream=stream,
            stream_options=utils.get_pydantic_model(
                stream_options, OptionalNullable[components.ChatStreamOptions]
            ),
            temperature=temperature,
            tool_choice=tool_choice,
            tools=utils.get_pydantic_model(
                tools, Optional[List[components.ToolDefinitionJSON]]
            ),
            top_p=top_p,
            debug=utils.get_pydantic_model(debug, Optional[components.Debug]),
            image_config=image_config,
            modalities=modalities,
        )

        req = self._build_request(
            method="POST",
            path="/chat/completions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream" if stream else "application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.ChatGenerationParams
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="sendChatCompletionRequest",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "429", "4XX", "500", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            return unmarshal_json_response(
                components.ChatResponse, http_res, http_res_text
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStream(
                http_res,
                lambda raw: utils.unmarshal_json(
                    raw, components.ChatStreamingResponseChunk
                ).data,
                sentinel="[DONE]",
                client_ref=self,
            )
        if utils.match_response(http_res, ["400", "401", "429"], "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.ChatErrorData, http_res, http_res_text
            )
            raise errors.ChatError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.ChatErrorData, http_res, http_res_text
            )
            raise errors.ChatError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        http_res_text = utils.stream_to_text(http_res)
        raise errors.OpenRouterDefaultError(
            "Unexpected response received", http_res, http_res_text
        )

    @overload
    async def send_async(
        self,
        *,
        messages: Union[List[components.Message], List[components.MessageTypedDict]],
        provider: OptionalNullable[
            Union[
                components.ChatGenerationParamsProvider,
                components.ChatGenerationParamsProviderTypedDict,
            ]
        ] = UNSET,
        plugins: Optional[
            Union[
                List[components.ChatGenerationParamsPluginUnion],
                List[components.ChatGenerationParamsPluginUnionTypedDict],
            ]
        ] = None,
        route: OptionalNullable[components.Route] = UNSET,
        user: Optional[str] = None,
        session_id: Optional[str] = None,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[Dict[str, float]] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        top_logprobs: OptionalNullable[float] = UNSET,
        max_completion_tokens: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[float] = UNSET,
        metadata: Optional[Dict[str, str]] = None,
        presence_penalty: OptionalNullable[float] = UNSET,
        reasoning: Optional[
            Union[components.Reasoning, components.ReasoningTypedDict]
        ] = None,
        response_format: Optional[
            Union[
                components.ChatGenerationParamsResponseFormatUnion,
                components.ChatGenerationParamsResponseFormatUnionTypedDict,
            ]
        ] = None,
        seed: OptionalNullable[int] = UNSET,
        stop: OptionalNullable[
            Union[
                components.ChatGenerationParamsStop,
                components.ChatGenerationParamsStopTypedDict,
            ]
        ] = UNSET,
        stream: Union[Literal[False], None] = None,
        stream_options: OptionalNullable[
            Union[components.ChatStreamOptions, components.ChatStreamOptionsTypedDict]
        ] = UNSET,
        temperature: OptionalNullable[float] = UNSET,
        tool_choice: Optional[Any] = None,
        tools: Optional[
            Union[
                List[components.ToolDefinitionJSON],
                List[components.ToolDefinitionJSONTypedDict],
            ]
        ] = None,
        top_p: OptionalNullable[float] = UNSET,
        debug: Optional[Union[components.Debug, components.DebugTypedDict]] = None,
        image_config: Optional[
            Union[
                Dict[str, components.ChatGenerationParamsImageConfig],
                Dict[str, components.ChatGenerationParamsImageConfigTypedDict],
            ]
        ] = None,
        modalities: Optional[List[components.Modality]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> components.ChatResponse:
        r"""Create a chat completion

        Sends a request for a model response for the given chat conversation. Supports both streaming and non-streaming modes.

        :param messages:
        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param route:
        :param user:
        :param session_id: A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 128 characters.
        :param model:
        :param models:
        :param frequency_penalty:
        :param logit_bias:
        :param logprobs:
        :param top_logprobs:
        :param max_completion_tokens:
        :param max_tokens:
        :param metadata:
        :param presence_penalty:
        :param reasoning:
        :param response_format:
        :param seed:
        :param stop:
        :param stream:
        :param stream_options:
        :param temperature:
        :param tool_choice:
        :param tools:
        :param top_p:
        :param debug:
        :param image_config:
        :param modalities:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    @overload
    async def send_async(
        self,
        *,
        messages: Union[List[components.Message], List[components.MessageTypedDict]],
        provider: OptionalNullable[
            Union[
                components.ChatGenerationParamsProvider,
                components.ChatGenerationParamsProviderTypedDict,
            ]
        ] = UNSET,
        plugins: Optional[
            Union[
                List[components.ChatGenerationParamsPluginUnion],
                List[components.ChatGenerationParamsPluginUnionTypedDict],
            ]
        ] = None,
        route: OptionalNullable[components.Route] = UNSET,
        user: Optional[str] = None,
        session_id: Optional[str] = None,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[Dict[str, float]] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        top_logprobs: OptionalNullable[float] = UNSET,
        max_completion_tokens: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[float] = UNSET,
        metadata: Optional[Dict[str, str]] = None,
        presence_penalty: OptionalNullable[float] = UNSET,
        reasoning: Optional[
            Union[components.Reasoning, components.ReasoningTypedDict]
        ] = None,
        response_format: Optional[
            Union[
                components.ChatGenerationParamsResponseFormatUnion,
                components.ChatGenerationParamsResponseFormatUnionTypedDict,
            ]
        ] = None,
        seed: OptionalNullable[int] = UNSET,
        stop: OptionalNullable[
            Union[
                components.ChatGenerationParamsStop,
                components.ChatGenerationParamsStopTypedDict,
            ]
        ] = UNSET,
        stream: Literal[True],
        stream_options: OptionalNullable[
            Union[components.ChatStreamOptions, components.ChatStreamOptionsTypedDict]
        ] = UNSET,
        temperature: OptionalNullable[float] = UNSET,
        tool_choice: Optional[Any] = None,
        tools: Optional[
            Union[
                List[components.ToolDefinitionJSON],
                List[components.ToolDefinitionJSONTypedDict],
            ]
        ] = None,
        top_p: OptionalNullable[float] = UNSET,
        debug: Optional[Union[components.Debug, components.DebugTypedDict]] = None,
        image_config: Optional[
            Union[
                Dict[str, components.ChatGenerationParamsImageConfig],
                Dict[str, components.ChatGenerationParamsImageConfigTypedDict],
            ]
        ] = None,
        modalities: Optional[List[components.Modality]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStreamAsync[components.ChatStreamingResponseChunkData]:
        r"""Create a chat completion

        Sends a request for a model response for the given chat conversation. Supports both streaming and non-streaming modes.

        :param messages:
        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param route:
        :param user:
        :param session_id: A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 128 characters.
        :param model:
        :param models:
        :param frequency_penalty:
        :param logit_bias:
        :param logprobs:
        :param top_logprobs:
        :param max_completion_tokens:
        :param max_tokens:
        :param metadata:
        :param presence_penalty:
        :param reasoning:
        :param response_format:
        :param seed:
        :param stop:
        :param stream:
        :param stream_options:
        :param temperature:
        :param tool_choice:
        :param tools:
        :param top_p:
        :param debug:
        :param image_config:
        :param modalities:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    async def send_async(
        self,
        *,
        messages: Union[List[components.Message], List[components.MessageTypedDict]],
        provider: OptionalNullable[
            Union[
                components.ChatGenerationParamsProvider,
                components.ChatGenerationParamsProviderTypedDict,
            ]
        ] = UNSET,
        plugins: Optional[
            Union[
                List[components.ChatGenerationParamsPluginUnion],
                List[components.ChatGenerationParamsPluginUnionTypedDict],
            ]
        ] = None,
        route: OptionalNullable[components.Route] = UNSET,
        user: Optional[str] = None,
        session_id: Optional[str] = None,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[Dict[str, float]] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        top_logprobs: OptionalNullable[float] = UNSET,
        max_completion_tokens: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[float] = UNSET,
        metadata: Optional[Dict[str, str]] = None,
        presence_penalty: OptionalNullable[float] = UNSET,
        reasoning: Optional[
            Union[components.Reasoning, components.ReasoningTypedDict]
        ] = None,
        response_format: Optional[
            Union[
                components.ChatGenerationParamsResponseFormatUnion,
                components.ChatGenerationParamsResponseFormatUnionTypedDict,
            ]
        ] = None,
        seed: OptionalNullable[int] = UNSET,
        stop: OptionalNullable[
            Union[
                components.ChatGenerationParamsStop,
                components.ChatGenerationParamsStopTypedDict,
            ]
        ] = UNSET,
        stream: Optional[bool] = False,
        stream_options: OptionalNullable[
            Union[components.ChatStreamOptions, components.ChatStreamOptionsTypedDict]
        ] = UNSET,
        temperature: OptionalNullable[float] = UNSET,
        tool_choice: Optional[Any] = None,
        tools: Optional[
            Union[
                List[components.ToolDefinitionJSON],
                List[components.ToolDefinitionJSONTypedDict],
            ]
        ] = None,
        top_p: OptionalNullable[float] = UNSET,
        debug: Optional[Union[components.Debug, components.DebugTypedDict]] = None,
        image_config: Optional[
            Union[
                Dict[str, components.ChatGenerationParamsImageConfig],
                Dict[str, components.ChatGenerationParamsImageConfigTypedDict],
            ]
        ] = None,
        modalities: Optional[List[components.Modality]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.SendChatCompletionRequestResponse:
        r"""Create a chat completion

        Sends a request for a model response for the given chat conversation. Supports both streaming and non-streaming modes.

        :param messages:
        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param route:
        :param user:
        :param session_id: A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 128 characters.
        :param model:
        :param models:
        :param frequency_penalty:
        :param logit_bias:
        :param logprobs:
        :param top_logprobs:
        :param max_completion_tokens:
        :param max_tokens:
        :param metadata:
        :param presence_penalty:
        :param reasoning:
        :param response_format:
        :param seed:
        :param stop:
        :param stream:
        :param stream_options:
        :param temperature:
        :param tool_choice:
        :param tools:
        :param top_p:
        :param debug:
        :param image_config:
        :param modalities:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = components.ChatGenerationParams(
            provider=utils.get_pydantic_model(
                provider, OptionalNullable[components.ChatGenerationParamsProvider]
            ),
            plugins=utils.get_pydantic_model(
                plugins, Optional[List[components.ChatGenerationParamsPluginUnion]]
            ),
            route=route,
            user=user,
            session_id=session_id,
            messages=utils.get_pydantic_model(messages, List[components.Message]),
            model=model,
            models=models,
            frequency_penalty=frequency_penalty,
            logit_bias=logit_bias,
            logprobs=logprobs,
            top_logprobs=top_logprobs,
            max_completion_tokens=max_completion_tokens,
            max_tokens=max_tokens,
            metadata=metadata,
            presence_penalty=presence_penalty,
            reasoning=utils.get_pydantic_model(
                reasoning, Optional[components.Reasoning]
            ),
            response_format=utils.get_pydantic_model(
                response_format,
                Optional[components.ChatGenerationParamsResponseFormatUnion],
            ),
            seed=seed,
            stop=stop,
            stream=stream,
            stream_options=utils.get_pydantic_model(
                stream_options, OptionalNullable[components.ChatStreamOptions]
            ),
            temperature=temperature,
            tool_choice=tool_choice,
            tools=utils.get_pydantic_model(
                tools, Optional[List[components.ToolDefinitionJSON]]
            ),
            top_p=top_p,
            debug=utils.get_pydantic_model(debug, Optional[components.Debug]),
            image_config=image_config,
            modalities=modalities,
        )

        req = self._build_request_async(
            method="POST",
            path="/chat/completions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream" if stream else "application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.ChatGenerationParams
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="sendChatCompletionRequest",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "429", "4XX", "500", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            return unmarshal_json_response(
                components.ChatResponse, http_res, http_res_text
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStreamAsync(
                http_res,
                lambda raw: utils.unmarshal_json(
                    raw, components.ChatStreamingResponseChunk
                ).data,
                sentinel="[DONE]",
                client_ref=self,
            )
        if utils.match_response(http_res, ["400", "401", "429"], "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.ChatErrorData, http_res, http_res_text
            )
            raise errors.ChatError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.ChatErrorData, http_res, http_res_text
            )
            raise errors.ChatError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.OpenRouterDefaultError(
            "Unexpected response received", http_res, http_res_text
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .defaultparameters import DefaultParameters, DefaultParametersTypedDict
from .modelarchitecture import ModelArchitecture, ModelArchitectureTypedDict
from .parameter import Parameter
from .perrequestlimits import PerRequestLimits, PerRequestLimitsTypedDict
from .publicpricing import PublicPricing, PublicPricingTypedDict
from .topproviderinfo import TopProviderInfo, TopProviderInfoTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ModelTypedDict(TypedDict):
    r"""Information about an AI model available on OpenRouter"""

    id: str
    r"""Unique identifier for the model"""
    canonical_slug: str
    r"""Canonical slug for the model"""
    name: str
    r"""Display name of the model"""
    created: float
    r"""Unix timestamp of when the model was created"""
    pricing: PublicPricingTypedDict
    r"""Pricing information for the model"""
    context_length: Nullable[float]
    r"""Maximum context length in tokens"""
    architecture: ModelArchitectureTypedDict
    r"""Model architecture information"""
    top_provider: TopProviderInfoTypedDict
    r"""Information about the top provider for this model"""
    per_request_limits: Nullable[PerRequestLimitsTypedDict]
    r"""Per-request token limits"""
    supported_parameters: List[Parameter]
    r"""List of supported parameters for this model"""
    default_parameters: Nullable[DefaultParametersTypedDict]
    r"""Default parameters for this model"""
    hugging_face_id: NotRequired[Nullable[str]]
    r"""Hugging Face model identifier, if applicable"""
    description: NotRequired[str]
    r"""Description of the model"""


class Model(BaseModel):
    r"""Information about an AI model available on OpenRouter"""

    id: str
    r"""Unique identifier for the model"""

    canonical_slug: str
    r"""Canonical slug for the model"""

    name: str
    r"""Display name of the model"""

    created: float
    r"""Unix timestamp of when the model was created"""

    pricing: PublicPricing
    r"""Pricing information for the model"""

    context_length: Nullable[float]
    r"""Maximum context length in tokens"""

    architecture: ModelArchitecture
    r"""Model architecture information"""

    top_provider: TopProviderInfo
    r"""Information about the top provider for this model"""

    per_request_limits: Nullable[PerRequestLimits]
    r"""Per-request token limits"""

    supported_parameters: List[
        Annotated[Parameter, PlainValidator(validate_open_enum(False))]
    ]
    r"""List of supported parameters for this model"""

    default_parameters: Nullable[DefaultParameters]
    r"""Default parameters for this model"""

    hugging_face_id: OptionalNullable[str] = UNSET
    r"""Hugging Face model identifier, if applicable"""

    description: Optional[str] = None
    r"""Description of the model"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["hugging_face_id", "description"]
        nullable_fields = [
            "hugging_face_id",
            "context_length",
            "per_request_limits",
            "default_parameters",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


ExchangeAuthCodeForAPIKeyCodeChallengeMethod = Union[
    Literal[
        "S256",
        "plain",
    ],
    UnrecognizedStr,
]
r"""The method used to generate the code challenge"""


class ExchangeAuthCodeForAPIKeyRequestTypedDict(TypedDict):
    code: str
    r"""The authorization code received from the OAuth redirect"""
    code_verifier: NotRequired[str]
    r"""The code verifier if code_challenge was used in the authorization request"""
    code_challenge_method: NotRequired[
        Nullable[ExchangeAuthCodeForAPIKeyCodeChallengeMethod]
    ]
    r"""The method used to generate the code challenge"""


class ExchangeAuthCodeForAPIKeyRequest(BaseModel):
    code: str
    r"""The authorization code received from the OAuth redirect"""

    code_verifier: Optional[str] = None
    r"""The code verifier if code_challenge was used in the authorization request"""

    code_challenge_method: Annotated[
        OptionalNullable[ExchangeAuthCodeForAPIKeyCodeChallengeMethod],
        PlainValidator(validate_open_enum(False)),
    ] = UNSET
    r"""The method used to generate the code challenge"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["code_verifier", "code_challenge_method"]
        nullable_fields = ["code_challenge_method"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ExchangeAuthCodeForAPIKeyResponseTypedDict(TypedDict):
    r"""Successfully exchanged code for an API key"""

    key: str
    r"""The API key to use for OpenRouter requests"""
    user_id: Nullable[str]
    r"""User ID associated with the API key"""


class ExchangeAuthCodeForAPIKeyResponse(BaseModel):
    r"""Successfully exchanged code for an API key"""

    key: str
    r"""The API key to use for OpenRouter requests"""

    user_id: Nullable[str]
    r"""User ID associated with the API key"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["user_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
